from setuptools import setup

setup(
        name='mail-mojo',
        version='0.1.2',
        packages=['mail_mojo'],
        url='',
        license='GPL',
        author='Kelly',
        author_email='',
        description='',
        install_requires=['exchangelib'],
)
